@@ -3,6 +3,11 @@
 /* Definitions and Utilities needed by Runge-Kutta Solvers                  */
 /*==========================================================================*/
 
+/* USE_FC_LEN_T to ensure compatibility with Fortran BLAS/LAPACK */
+#ifndef USE_FC_LEN_T
+# define USE_FC_LEN_T
+#endif
+
 /* Load headers needed by the R interface */
 #include <R_ext/Rdynload.h>
 #include <R_ext/Applic.h> /* for dgemm */
@@ -15,7 +20,7 @@
 # define LDOUBLE double
 #endif
 
-
+#include "externalptr.h"
 
 /*============================================================================*/
 /*   DLL specific functions                                                   */
@@ -62,17 +67,16 @@ SEXP getInputs(SEXP symbol, SEXP Rho) {
 /*----------------------------------------------------------------------------*/
 
 void blas_matprod1(double *x, int nrx, int ncx,
-		    double *y, int nry, int ncy, double *z) 
-{
+      double *y, int nry, int ncy, double *z) {
     const char *transa = "N", *transb = "N";
     int i;
     double one = 1.0, zero = 0.0;
 
     if (nrx > 0 && ncx > 0 && nry > 0 && ncy > 0) {
-	    F77_CALL(dgemm)(transa, transb, &nrx, &ncy, &ncx, &one,
-			    x, &nrx, y, &nry, &zero, z, &nrx);
+      F77_CALL(dgemm)(transa, transb, &nrx, &ncy, &ncx, &one,
+          x, &nrx, y, &nry, &zero, z, &nrx FCONE FCONE);
     } else /* zero-extent operations should return zeroes */
-    	for(i = 0; i < nrx*ncy; i++) z[i] = 0;
+      for(i = 0; i < nrx*ncy; i++) z[i] = 0;
 }
 
 
@@ -81,10 +85,10 @@ void matprod(int m, int n, int o, double* a, double* b, double* c) {
   int i, j, k;
   for (i = 0; i < m; i++) {
     for (j = 0; j < o; j++) {
-    	c[i + m * j] = 0;
-    	for (k = 0; k < n; k++) {
-    	  c[i + m * j] += a[i + m * k] * b[k + n * j];
-    	}
+      c[i + m * j] = 0;
+      for (k = 0; k < n; k++) {
+        c[i + m * j] += a[i + m * k] * b[k + n * j];
+      }
     }
   }
 }
@@ -110,7 +114,7 @@ double maxerr(double *y0, double *y1, double *y2, double *Atol, double *Rtol, in
 /*   CALL TO THE MODEL FUNCTION                                             */
 /*==========================================================================*/
 void derivs(SEXP Func, double t, double* y, SEXP Parms, SEXP Rho,
-	    double *ydot, double *yout, int j, int neq, int *ipar, int isDll,
+      double *ydot, double *yout, int j, int neq, int *ipar, int isDll,
             int isForcing) {
   SEXP Val, rVal, R_fcall;
   SEXP R_t;
@@ -125,8 +129,8 @@ void derivs(SEXP Func, double t, double* y, SEXP Parms, SEXP Rho,
     /*   Function is a DLL function                                           */
     /*------------------------------------------------------------------------*/
     C_deriv_func_type *cderivs;
-    if (isForcing) updatedeforc(&t); 
-    cderivs = (C_deriv_func_type *) R_ExternalPtrAddr(Func);
+    if (isForcing) updatedeforc(&t);
+    cderivs = (C_deriv_func_type *) R_ExternalPtrAddrFn_(Func);
     cderivs(&neq, &t, y, ytmp, yout, ipar);
     if (j >= 0)
       for (i = 0; i < neq; i++)  ydot[i + neq * j] = ytmp[i];
@@ -134,17 +138,30 @@ void derivs(SEXP Func, double t, double* y, SEXP Parms, SEXP Rho,
     /*------------------------------------------------------------------------*/
     /* Function is an R function                                              */
     /*------------------------------------------------------------------------*/
-    PROTECT(R_t = ScalarReal(t)); incr_N_Protect();
-    PROTECT(R_y = allocVector(REALSXP, neq)); incr_N_Protect();
+    PROTECT(R_t = ScalarReal(t));
+    PROTECT(R_y = allocVector(REALSXP, neq));
     yy = REAL(R_y);
-    for (i=0; i< neq; i++) yy[i] = y[i];
+    for (i=0; i < neq; i++) yy[i] = y[i];
 
-    PROTECT(R_fcall = lang4(Func, R_t, R_y, Parms)); incr_N_Protect();
-    PROTECT(Val = eval(R_fcall, Rho)); incr_N_Protect();
+    PROTECT(R_fcall = lang4(Func, R_t, R_y, Parms));
+    PROTECT(Val = eval(R_fcall, Rho));
 
     /* extract the states from first list element of "Val" */
+
+    /* original version up to deSolve 1.28
     if (j >= 0)
       for (i = 0; i < neq; i++)  ydot[i + neq * j] = REAL(VECTOR_ELT(Val, 0))[i];
+    */
+
+    /* allow states to be integer (useful for discrete models) deSolve 1.29 */
+    if (j >= 0) {
+      PROTECT(rVal = coerceVector(VECTOR_ELT(Val, 0), REALSXP));
+      for (i = 0; i < neq; i++)  {
+        ydot[i + neq * j] = REAL(rVal)[i];
+      }
+      UNPROTECT(1);
+    }
+
 
     /* extract outputs from second and following list elements */
     /* this is essentially an unlist for non-nested numeric lists */
@@ -153,9 +170,8 @@ void derivs(SEXP Func, double t, double* y, SEXP Parms, SEXP Rho,
       for (i = 0; i < nout; i++)  {
         l = LENGTH(VECTOR_ELT(Val, elt));
         if (ii == l) {
-	        ii = 0; elt++;
-	      }
-        //yout[i] = REAL(VECTOR_ELT(Val, elt))[ii];
+          ii = 0; elt++;
+        }
         // thpe 2012-08-04: make sure the return value is double and not int
         PROTECT(rVal = coerceVector(VECTOR_ELT(Val, elt), REALSXP));
         yout[i] = REAL(rVal)[ii];
@@ -163,7 +179,7 @@ void derivs(SEXP Func, double t, double* y, SEXP Parms, SEXP Rho,
         ii++;
       }
     }
-    my_unprotect(4);
+    UNPROTECT(4);
   }
 }
 
@@ -180,16 +196,16 @@ void denspar(double *FF, double *y0, double *y1, double dt, double *d,
   double ydiff, bspl;
   int i, j;
   for (i = 0; i < neq; i++) {
-   r[i]           = y0[i];
-   ydiff          = y1[i] - y0[i];
-   r[i + neq]     = ydiff;
-   bspl           = dt * FF[i] - ydiff;
-   r[i + 2 * neq] = bspl;
-   r[i + 3 * neq] = ydiff - dt * FF[i + (stage - 1) * neq] - bspl;
-   r[i + 4 * neq] = 0;
-   for (j = 0; j < stage; j++)
-     r[i + 4 * neq] = r[i + 4 * neq] + d[j] * FF[i + j * neq];
-     r[i + 4 * neq] = r[i + 4 * neq] * dt;
+    r[i]           = y0[i];
+    ydiff          = y1[i] - y0[i];
+    r[i + neq]     = ydiff;
+    bspl           = dt * FF[i] - ydiff;
+    r[i + 2 * neq] = bspl;
+    r[i + 3 * neq] = ydiff - dt * FF[i + (stage - 1) * neq] - bspl;
+    r[i + 4 * neq] = 0;
+    for (j = 0; j < stage; j++)
+      r[i + 4 * neq] = r[i + 4 * neq] + d[j] * FF[i + j * neq];
+    r[i + 4 * neq] = r[i + 4 * neq] * dt;
   }
 }
 
@@ -202,10 +218,10 @@ void densout(double *r, double t0, double t, double dt, double* res, int neq) {
 }
 
 /*----------------------------------------------------------------------------*/
-/* dense output for the Cash-Karp method  - does not work (yet)              */
+/* dense output for the Cash-Karp method  - does not work (yet)               */
 /*----------------------------------------------------------------------------*/
 
-void densoutck(double t0, double t, double dt, double* y0,  
+void densoutck(double t0, double t, double dt, double* y0,
  double* FF, double* dy, double* res, int neq) {
 
   double s, s2, s3, s4, b1, b3, b4, b5, b6, b7;
@@ -213,17 +229,17 @@ void densoutck(double t0, double t, double dt, double* y0,
   s2 = s  * s;
   s3 = s2 * s;
   s4 = s3 * s;
-  
+
   b3 = 500./161.    * s2 - 20000./4347.* s3 + 2750./1449.* s4;
   b4 = 125./132.    * s2 - 625./594.   * s3 + 125./396.  * s4;
   b5 = 15./28.      * s2 - 15./14.     * s3 + 15./28.    * s4;
   b6 = -6144./1771. * s2 + 2048./253.  * s3 - 7680./1771.* s4;
-  b7 = 3./2.        * s2 - 4.          * s3 + 5./2.      * s4;   
-  
+  b7 = 3./2.        * s2 - 4.          * s3 + 5./2.      * s4;
+
   b1 = s-b3-b4-b5-b6-b7;
 
   for (int i = 0; i < neq; i++)
-    res[i] = y0[i] + b1 * dt * FF[i + 0 * neq] + b3 * dt * FF[i + 2 * neq] 
+    res[i] = y0[i] + b1 * dt * FF[i + 0 * neq] + b3 * dt * FF[i + 2 * neq]
                    + b4 * dt * FF[i + 3 * neq] + b5 * dt * FF[i + 4 * neq]
                    + b6 * dt * FF[i + 5 * neq] + b7 * dt * dy[i];
 }
@@ -232,8 +248,8 @@ void densoutck(double t0, double t, double dt, double* y0,
 /* Polynomial interpolation                                                   */
 /*    ksig: number of signals                                                 */
 /*    n:    number of knots per signal                                        */
-/*    x[0 .. n-1]:          vector of x values                                */
-/*    y[0 .. n-1, 0 .. ksig] array  of y values                               */
+/*    x[0 .. n-1]:            vector of x values                              */
+/*    y[0 .. n-1, 0 .. ksig]: array  of y values                              */
 /*----------------------------------------------------------------------------*/
 void neville(double *xx, double *y, double tnew, double *ynew, int n, int ksig) {
   int i, j, k;
@@ -272,9 +288,9 @@ void setIstate(SEXP R_yout, SEXP R_istate, int *istate,
   /* note that indices are 1 smaller in C than in R  */
   istate[11] = it_tot;                      /* number of steps */
   istate[12] = it_tot * (stage - fsal) + 1; /* number of function evaluations */
-  if (fsal) 
-    istate[12] = istate[12] + nrej + 1;     /* one more ftion eval if rejected*/
-  istate[13] = nrej;                        /* number of rejected steps */  
+  if (fsal)                                 /* first same as last */
+    istate[12] = istate[12] + nrej + 1;     /* one more fct. eval if rejected*/
+  istate[13] = nrej;                        /* number of rejected steps */
   istate[14] = qerr;                        /* order of the method */
   setAttrib(R_yout, install("istate"), R_istate);
 }
